import java.io.InputStream;
import java.io.FileInputStream;
import EBNFTokens._;
  
class EBNFParser (in: InputStream) extends EBNFScanner(in) {
  nextToken;
  /** syntax = {production} <eof>. */
  def syntax: Unit = 
    while (token != EOF) production;
  /** production = identifier "=" expression ".". */
  def production: Unit = {
    if (token == IDENT) nextToken 
    else error("illegal start of production");
    if (token == EQL) nextToken
    else error("'=' expected");
    expression;
    if (token == PERIOD) nextToken
    else error("'.' expected");
  }
  /** expression = term {"|" term}. */
  def expression: Unit = {
    term;
    while (token == BAR) {
      nextToken;
      term;
  }}
  /** term = {factor}. */
  def term: Unit =
    while (List(IDENT,LITERAL,LPAREN,LBRACK,LBRACE).contains(token)) factor;
  /** factor = identifier | string | "(" expression ")" | "[" expression "]" | "{" expression "}". */
  def factor: Unit = token match {
    case IDENT => nextToken;
    case LITERAL => nextToken;
    case LPAREN => {
      nextToken; expression;
      if (token == RPAREN) nextToken
      else error("')' expected");
    }
    case LBRACK => {
      nextToken; expression;
      if (token == RBRACK) nextToken
      else error("']' expected");
    }
    case LBRACE => {
      nextToken; expression;
      if (token == RBRACE) nextToken
      else error("'}' expected");
    }
    case _ => error("illegal start of factor");
}}

object EBNFParserTest {
  def main (args: Array[String]) = {
    val scan = new EBNFParser(new FileInputStream(args(0)));
    scan.syntax;
    scan.close;
    Console.println("analyse syntaxique et lexicale réussie !");
  }
}