import scala.collection.mutable._;

class FFNCalculator {

  type Symbol = String;
  type Production = Pair[Symbol, List[Symbol]];

  def getTnT (productions: List[Production]): Pair[Set[Symbol], Set[Symbol]] = {
    var terminals = new HashSet[Symbol]();
    var nonTerminals = new HashSet[Symbol]();
    for (val Production(left, right) <- productions) {
      nonTerminals += left;
      terminals += left; terminals ++= right;
    }
    terminals.filter(symbl => !nonTerminals.contains(symbl));
    Pair(terminals, nonTerminals)
  }

  def takeRange[A](xs: List[A], from: Int, to: Int): List[A] =
    xs.take(to).takeRight(to - from);

  def calculate (productions: List[Production]): Triple[HashMap[Symbol,HashSet[Symbol]], HashMap[Symbol,HashSet[Symbol]], HashSet[Symbol]] = {
    val Pair(terminals, nonTerminals) = getTnT(productions);
    val nullable = new HashSet[Symbol]();
    val first = new HashMap[Symbol,HashSet[Symbol]]();
    val follow = new HashMap[Symbol,HashSet[Symbol]]();
    for (val term <- terminals) { first(term) = new HashSet(); first(term) += term; follow(term) = new HashSet() }
    for (val nonTerm <- nonTerminals) { first(nonTerm) = new HashSet(); follow(nonTerm) = new HashSet() }

    for (val term <- terminals) first(term) += term;
    def allNullable(xs: List[Symbol]) = xs.forall(r => nullable.contains(r));
    var fixpointReached = false;
    do {
      val nullableCheck = nullable; val firstCheck = first; val followCheck = follow;
      for (val Production(left, right) <- productions) {
        val k = right.length;
        if (allNullable(right)) nullable += left;
        for (val i <- List.range(0, k))
          if (allNullable(takeRange(right, 0, i)))
            first(left) ++= first(right(i));
        for (val i <- List.range(0, k))
          if (allNullable(takeRange(right, i+1, k)))
            follow(right(i)) ++= follow(left);
        for (val i <- List.range(0, k); val j <- List.range(i+1, k))
          if (allNullable(takeRange(right, i+1, j)))
            follow(right(i)) ++= first(right(j));
      }
      fixpointReached = nullableCheck == nullable && firstCheck == first && followCheck == follow;
    } while (!fixpointReached);
    Triple(first, follow, nullable)
  }
}

object FFNCalculatorTest {

  def main (args: Array[String]): Unit = {
    val ffnc = new FFNCalculator;
    val grammar = List(
        Pair("S", List("a", "A", "c")),
        Pair("A", Nil),
        Pair("A", List("b"))
      );
    val Triple(first, follow, nullable) = ffnc.calculate(grammar);
    Console.println("first: "+first);
    Console.println("follow: "+follow);
    Console.println("nullable: "+nullable);
  }
}
